@@ -7,13 +7,10 @@ use brunch::{
 	benches,
 };
 use dowser::Dowser;
-use std::{
-	path::PathBuf,
-	time::Duration,
-};
+use std::time::Duration;
 
 benches!(
 	Bench::new("dowser::Dowser", "with_path(/usr/share)")
 		.timed(Duration::from_secs(6))
-		.with(|| Vec::<PathBuf>::try_from(Dowser::default().with_path("/usr/share")))
+		.with(|| Dowser::default().with_path("/usr/share").into_vec()),
 );
